
#include <windows.h>  
#include "server.h"  
#include "time.h"


#if defined (WIN32)
	#define IS_WIN32 TRUE
#else
	#define IS_WIN32 FALSE
#endif

#define IS_NT      IS_WIN32 && (BOOL)(GetVersion() < 0x80000000)
#define IS_WIN32S  IS_WIN32 && (BOOL)(!(IS_NT) && (LOBYTE(LOWORD(GetVersion()))<4))
#define IS_WIN95   (BOOL)(!(IS_NT) && !(IS_WIN32S)) && IS_WIN32

HINSTANCE hInst;   // current instance

LPCTSTR lpszAppName = "MyApp";
LPCTSTR lpszTitle   = "DDE Server"; 


BOOL RegisterWin95( CONST WNDCLASS* lpwc );


int APIENTRY WinMain( HINSTANCE hInstance, HINSTANCE hPrevInstance,
                      LPTSTR lpCmdLine, int nCmdShow)
{
   MSG      msg;
   HWND     hWnd; 
   WNDCLASS wc;

   wc.style         = CS_HREDRAW | CS_VREDRAW;
   wc.lpfnWndProc   = (WNDPROC)WndProc;       
   wc.cbClsExtra    = 0;                      
   wc.cbWndExtra    = 0;                      
   wc.hInstance     = hInstance;              
   wc.hIcon         = LoadIcon (hInstance, lpszAppName); 
   wc.hCursor       = LoadCursor(NULL, IDC_ARROW);
   wc.hbrBackground = (HBRUSH)(COLOR_WINDOW+1);
   wc.lpszMenuName  = lpszAppName;              
   wc.lpszClassName = lpszAppName;              

   if ( IS_WIN95 )
   {
      if ( !RegisterWin95( &wc ) )
         return( FALSE );
   }
   else if ( !RegisterClass( &wc ) )
      return( FALSE );

   hInst = hInstance; 

   hWnd = CreateWindow( lpszAppName, 
                        lpszTitle,    
                        WS_OVERLAPPEDWINDOW, 
                        CW_USEDEFAULT, 0, 
                        CW_USEDEFAULT, 0,  
                        NULL,              
                        NULL,              
                        hInstance,         
                        NULL               
                      );

   if ( !hWnd ) 
      return( FALSE );

   ShowWindow( hWnd, nCmdShow ); 
   UpdateWindow( hWnd );         

   while( GetMessage( &msg, NULL, 0, 0) )   
   {
      TranslateMessage( &msg ); 
      DispatchMessage( &msg );  
   }

   return( msg.wParam ); 
}


BOOL RegisterWin95( CONST WNDCLASS* lpwc )
{
   WNDCLASSEX wcex;

   wcex.style         = lpwc->style;
   wcex.lpfnWndProc   = lpwc->lpfnWndProc;
   wcex.cbClsExtra    = lpwc->cbClsExtra;
   wcex.cbWndExtra    = lpwc->cbWndExtra;
   wcex.hInstance     = lpwc->hInstance;
   wcex.hIcon         = lpwc->hIcon;
   wcex.hCursor       = lpwc->hCursor;
   wcex.hbrBackground = lpwc->hbrBackground;
   wcex.lpszMenuName  = lpwc->lpszMenuName;
   wcex.lpszClassName = lpwc->lpszClassName;

   // Added elements for Windows 95.
   //...............................
   wcex.cbSize = sizeof(WNDCLASSEX);
   wcex.hIconSm = LoadImage(wcex.hInstance, lpwc->lpszClassName, 
                            IMAGE_ICON, 16, 16,
                            LR_DEFAULTCOLOR );
			
   return RegisterClassEx( &wcex );
}

HDDEDATA CALLBACK DdemlCallback( UINT     uType,
                                 UINT     uFmt,	
                                 HCONV    hconv,	
                                 HSZ      hsz1,	
                                 HSZ      hsz2,	
                                 HDDEDATA hdata,	
                                 DWORD    dwData1,
                                 DWORD    dwData2 );


DWORD ddeInst    = 0;
HSZ   hszService = NULL;
HSZ   hszTopic1  = NULL;
HSZ   hszTopic2  = NULL;
HSZ   hszTopic3  = NULL;
HSZ   hszItem    = NULL;

LRESULT CALLBACK WndProc( HWND hWnd, UINT uMsg, WPARAM wParam, LPARAM lParam )
{
   switch( uMsg )
   {
      case WM_CREATE :
              // Initialize DDEML.
              //..................
              if ( DdeInitialize( &ddeInst, DdemlCallback, 
                   APPCLASS_STANDARD, 0 ) != DMLERR_NO_ERROR )
              {
                 MessageBox( hWnd, "Could not initialize DDEML.", 
                             NULL, MB_OK | MB_ICONSTOP );
                 return( -1 );
              }

              // Allocate strings.
              //..................
              hszService = DdeCreateStringHandle( ddeInst,"Server",CP_WINANSI );
              hszTopic1  = DdeCreateStringHandle( ddeInst,"Data",CP_WINANSI );
              hszTopic2  = DdeCreateStringHandle( ddeInst,"Msg",CP_WINANSI ); 
              hszTopic3  = DdeCreateStringHandle( ddeInst,"Time",CP_WINANSI );
              hszItem    = DdeCreateStringHandle( ddeInst,"TimeItem",CP_WINANSI );

              DdeNameService( ddeInst, hszService, 0L, DNS_REGISTER );
              break;

      case WM_COMMAND :
              switch( LOWORD( wParam ) )
              {
                 case IDM_ENABLE :
                        DdeEnableCallback( ddeInst, NULL, EC_ENABLEONE );
                        break;

                 case IDM_DISABLE :
                        DdeEnableCallback( ddeInst, NULL, EC_DISABLE );
                        break;

                 case IDM_ABOUT :
                        DialogBox( hInst, "AboutBox", hWnd, (DLGPROC)About );
                        break;

                 case IDM_EXIT :
                        DestroyWindow( hWnd );
                        break;
              }
              break;
      
      case WM_DESTROY :
              DdeNameService( ddeInst, hszService, 0L, DNS_UNREGISTER );

              DdeFreeStringHandle( ddeInst, hszService );
              DdeFreeStringHandle( ddeInst, hszTopic1 );
              DdeFreeStringHandle( ddeInst, hszTopic2 );
              DdeFreeStringHandle( ddeInst, hszTopic3 );
              DdeFreeStringHandle( ddeInst, hszItem );

              DdeUninitialize( ddeInst );
              PostQuitMessage(0);
              break;

      default :
            return( DefWindowProc( hWnd, uMsg, wParam, lParam ) );
   }

   return( 0L );
}


VOID CALLBACK TimeProc( HWND  hwnd, 
                        UINT  uMsg,
                        UINT  idEvent,
                        DWORD dwTime )
{
   DdePostAdvise( ddeInst, hszTopic3, hszItem );
}


HDDEDATA CALLBACK DdemlCallback( UINT     uType,
                                 UINT     uFmt,	
                                 HCONV    hconv,	
                                 HSZ      hsz1,	
                                 HSZ      hsz2,	
                                 HDDEDATA hdata,	
                                 DWORD    dwData1,
                                 DWORD    dwData2 )
{
static LPBYTE lpMsg    = NULL;
static UINT   uTimerID = 0;

   switch( uType )
   {
      case XTYP_CONNECT :
            if ( DdeCmpStringHandles( hsz2, hszService ) ) 
               break;

            if ( DdeCmpStringHandles( hsz1, hszTopic1 ) &&
                 DdeCmpStringHandles( hsz1, hszTopic2 ) &&
                 DdeCmpStringHandles( hsz1, hszTopic3 ) )
            {
               break;
            }

            // Allow the conversation.
            //........................
            return( (HDDEDATA)TRUE );

      case XTYP_REQUEST :
            if ( !DdeCmpStringHandles( hsz1, hszTopic1 ) )
            {
               HDDEDATA hData = DdeCreateDataHandle( ddeInst, NULL, 50, 0, 
                                                     hszTopic2, CF_TEXT, 0 );
               if ( hData )
               {
                  SHORT i, offs;
                  char  szTmp[3];

                  hData = DdeAddData( hData, "Data: ", 6, 0 );
                  offs = 6;

                  for( i=0; i<10; i++ )
                  {
                     itoa( i, szTmp, 10 );
                     hData = DdeAddData( hData, szTmp, strlen( szTmp ), offs );
                     offs += strlen( szTmp );
                  }
                  szTmp[0] = 0;
                  hData = DdeAddData( hData, szTmp, 1, offs );

                  return( hData );
               }
               return( NULL );
            }
            return( NULL );

             
      case XTYP_EXECUTE :
            if ( lpMsg )
               return( (HDDEDATA)DDE_FBUSY );

            if ( !DdeCmpStringHandles( hsz1, hszTopic2 ) )
            {
               lpMsg = DdeAccessData( hdata, NULL );

               MessageBox( NULL, lpMsg, "DDE Message", MB_OK );

               DdeUnaccessData( hdata );
               lpMsg = NULL;

               return( (HDDEDATA)DDE_FACK );
            }
            
            return( (HDDEDATA)DDE_FNOTPROCESSED );
      

      case XTYP_ADVSTART :
            if ( DdeCmpStringHandles( hsz1, hszTopic3 ) == 0 &&
                 DdeCmpStringHandles( hsz2, hszItem   ) == 0 )
            {
               uTimerID = SetTimer( NULL, 0, 1000, (TIMERPROC)TimeProc ); 
               return( (HDDEDATA)TRUE );
            }

            return( FALSE );

      case XTYP_ADVREQ :
            if ( DdeCmpStringHandles( hsz1, hszTopic3 ) == 0 &&
                 DdeCmpStringHandles( hsz2, hszItem   ) == 0 )
            {
               time_t   ltime;
               char     szBuf[128];
               HDDEDATA hDdeData;

               HSZ    hszDataName = DdeCreateStringHandle( ddeInst, 
                                            "Hour", CP_WINANSI );
               time( &ltime );
               wsprintf( szBuf, "The time is now: %s ", 
                                 (LPTSTR)ctime(&ltime) );

               hDdeData = DdeCreateDataHandle( ddeInst, szBuf,
                                     strlen( szBuf )+1, 0,
                                     hszDataName, CF_TEXT, 0 ); 

               DdeFreeStringHandle( ddeInst, hszDataName );
               return( hDdeData );
            }
            return( NULL );

      case XTYP_ADVSTOP :
            KillTimer( NULL, uTimerID );
            break;

   }

   return( NULL );
}


LRESULT CALLBACK About( HWND hDlg,           
                        UINT message,        
                        WPARAM wParam,       
                        LPARAM lParam)
{
   switch (message) 
   {
       case WM_INITDIALOG: 
               return (TRUE);

       case WM_COMMAND:                              
               if (   LOWORD(wParam) == IDOK         
                   || LOWORD(wParam) == IDCANCEL)    
               {
                       EndDialog(hDlg, TRUE);        
                       return (TRUE);
               }
               break;
   }

   return (FALSE); 
}
